@@ -17,14 +17,21 @@
 #include "Calculator.h"
 #include "MathStructure.h"
 #include "Number.h"
+#include "BuiltinFunctions.h"
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 #include <errno.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifdef COMPILED_DEFINITIONS
+#	include <gio/gio.h>
+#endif
 
-#define XML_GET_STRING_FROM_PROP(node, name, str)	value = xmlGetProp(node, (xmlChar*) name); if(value) {str = (char*) value; remove_blank_ends(str); xmlFree(value);} else str = ""; 
+using std::string;
+using std::vector;
+
+#define XML_GET_STRING_FROM_PROP(node, name, str)	value = xmlGetProp(node, (xmlChar*) name); if(value) {str = (char*) value; remove_blank_ends(str); xmlFree(value);} else str = "";
 #define XML_GET_STRING_FROM_TEXT(node, str)		value = xmlNodeListGetString(doc, node->xmlChildrenNode, 1); if(value) {str = (char*) value; remove_blank_ends(str); xmlFree(value);} else str = "";
 
 DataObject::DataObject(DataSet *parent_set) {
@@ -77,7 +84,7 @@ void DataObject::setNonlocalizedKeyProperty(DataProperty *property, string s_val
 	a_properties.push_back(-1);
 	s_nonlocalized_properties.push_back(s_value);
 }
-	
+
 const string &DataObject::getProperty(DataProperty *property, int *is_approximate) {
 	if(!property) return empty_string;
 	for(size_t i = 0; i < properties.size(); i++) {
@@ -132,7 +139,7 @@ bool DataObject::isUserModified() const {
 void DataObject::setUserModified(bool user_modified) {
 	b_uchanged = user_modified;
 }
-	
+
 DataSet *DataObject::parentSet() const {
 	return parent;
 }
@@ -163,10 +170,10 @@ void DataProperty::set(const DataProperty &dp) {
 	if(m_unit) m_unit->unref();
 	m_unit = NULL;
 	ptype = dp.propertyType();
-	b_key = dp.isKey(); 
-	b_case = dp.isCaseSensitive(); 
-	b_hide = dp.isHidden(); 
-	b_brackets = dp.usesBrackets(); 
+	b_key = dp.isKey();
+	b_case = dp.isCaseSensitive();
+	b_hide = dp.isHidden();
+	b_brackets = dp.usesBrackets();
 	b_approximate = dp.isApproximate();
 	b_uchanged = dp.isUserModified();
 	clearNames();
@@ -175,7 +182,7 @@ void DataProperty::set(const DataProperty &dp) {
 		name_is_ref.push_back(dp.nameIsReference(i));
 	}
 }
-	
+
 void DataProperty::setName(string s_name, bool is_ref) {
 	if(s_name.empty()) return;
 	names.clear();
@@ -365,7 +372,7 @@ DataSet::DataSet(const DataSet *o) {
 	b_loaded = false;
 	set(o);
 }
-	
+
 ExpressionItem *DataSet::copy() const {return new DataSet(this);}
 void DataSet::set(const ExpressionItem *item) {
 	if(item->type() == TYPE_FUNCTION && item->subtype() == SUBTYPE_DATA_SET) {
@@ -378,7 +385,7 @@ void DataSet::set(const ExpressionItem *item) {
 int DataSet::subtype() const {
 	return SUBTYPE_DATA_SET;
 }
-	
+
 int DataSet::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
 	DataObject *o = getObject(vargs[0]);
 	if(!o) {
@@ -507,7 +514,35 @@ bool DataSet::loadObjects(const char *file_name, bool is_user_defs) {
 	}
 	while(localebase.length() < 2) localebase += " ";
 
+#ifdef COMPILED_DEFINITIONS
+	if(strstr(file_name, "resource:") == file_name) {
+		doc = NULL;
+		GFile *f = g_file_new_for_uri(file_name);
+		if(f) {
+			GFileInputStream *s = g_file_read(f, NULL, NULL);
+			if(s) {
+				int res, size = 1024;
+				char chars[1024];
+				xmlParserCtxtPtr ctxt;
+				res = g_input_stream_read(G_INPUT_STREAM(s), chars, 4, NULL, NULL);
+				if(res > 0) {
+					ctxt = xmlCreatePushParserCtxt(NULL, NULL, chars, res, file_name);
+					while((res = g_input_stream_read(G_INPUT_STREAM(s), chars, size, NULL, NULL)) > 0) {
+						xmlParseChunk(ctxt, chars, res, 0);
+					}
+					xmlParseChunk(ctxt, chars, 0, 1);
+					doc = ctxt->myDoc;
+					xmlFreeParserCtxt(ctxt);
+				}
+			}
+		}
+	} else {
+		doc = xmlParseFile(file_name);
+	}
+#else
 	doc = xmlParseFile(file_name);
+#endif
+
 	if(doc == NULL) {
 		if(!is_user_defs && !isLocal()) {
 			CALCULATOR->error(true, _("Unable to load data objects in %s."), file_name, NULL);
@@ -590,7 +625,7 @@ bool DataSet::loadObjects(const char *file_name, bool is_user_defs) {
 							}
 						}
 					}
-				}	
+				}
 			}
 			old_object = false;
 			if(is_user_defs && objects_before > 0) {
@@ -607,7 +642,7 @@ bool DataSet::loadObjects(const char *file_name, bool is_user_defs) {
 				if(!old_object) o = new DataObject(this);
 				for(size_t i = 0; i < p_refs.size(); i++) {
 					o->setProperty(p_refs[i], s_refs[i]);
-				}	
+				}
 			}
 			child = cur->xmlChildrenNode;
 			while(child) {
@@ -622,7 +657,7 @@ bool DataSet::loadObjects(const char *file_name, bool is_user_defs) {
 							cmp = !xmlStrcmp(child->name, (const xmlChar*) properties[i]->getName(i2).c_str());
 						}
 						if(cmp) {
-							value = xmlGetProp(child, (xmlChar*) "approximate"); 
+							value = xmlGetProp(child, (xmlChar*) "approximate");
 							if(value) {
 								if(!xmlStrcmp(value, (const xmlChar*) "false")) {
 									i_approx = 0;
@@ -634,8 +669,8 @@ bool DataSet::loadObjects(const char *file_name, bool is_user_defs) {
 								i_approx = -1;
 							}
 							if(properties[i]->propertyType() == PROPERTY_STRING) {
-								value = xmlNodeListGetString(doc, child->xmlChildrenNode, 1); 
-								lang = xmlNodeGetLang(child); 
+								value = xmlNodeListGetString(doc, child->xmlChildrenNode, 1);
+								lang = xmlNodeGetLang(child);
 								ils = -1;
 								for(int i3 = lang_status_p.size() - 1; i3 > 0; i3--) {
 									if(lang_status_p[i3] == properties[i3]) {
@@ -658,12 +693,12 @@ bool DataSet::loadObjects(const char *file_name, bool is_user_defs) {
 											}
 										}
 									} else {
-										str = ""; 
+										str = "";
 									}
 									if(ils < 0 || lang_status[ils] < 1) {
-										o->setProperty(properties[i], str, i_approx); 
+										o->setProperty(properties[i], str, i_approx);
 									} else {
-										o->setNonlocalizedKeyProperty(properties[i], str); 
+										o->setNonlocalizedKeyProperty(properties[i], str);
 									}
 								} else if(!locale.empty() && (ils < 0 || lang_status[ils] < 2)) {
 									if(locale == (char*) lang) {
@@ -674,7 +709,7 @@ bool DataSet::loadObjects(const char *file_name, bool is_user_defs) {
 											lang_status[ils] = 2;
 										}
 										if(value) {
-											str = (char*) value; 
+											str = (char*) value;
 											remove_blank_ends(str);
 										} else {
 											str = "";
@@ -688,27 +723,27 @@ bool DataSet::loadObjects(const char *file_name, bool is_user_defs) {
 											lang_status[ils] = 1;
 										}
 										if(value) {
-											str = (char*) value; 
+											str = (char*) value;
 											remove_blank_ends(str);
 										} else {
 											str = "";
 										}
 										o->setProperty(properties[i], str, i_approx);
-									}	
-								} 
-								if(value) xmlFree(value); 
+									}
+								}
+								if(value) xmlFree(value);
 								if(lang) xmlFree(lang);
 							} else {
 								unc_rel = false;
 								uncertainty = xmlGetProp(child, (xmlChar*) "relative_uncertainty");
-								if(!uncertainty) uncertainty = xmlGetProp(child, (xmlChar*) "uncertainty"); 
+								if(!uncertainty) uncertainty = xmlGetProp(child, (xmlChar*) "uncertainty");
 								else unc_rel = true;
 								XML_GET_STRING_FROM_TEXT(child, str)
 								remove_blank_ends(str);
 								if(uncertainty) {
 									if(unc_rel) {
 										str.insert(0, "(");
-										str.insert(0, CALCULATOR->f_uncertainty->referenceName());
+										str.insert(0, CALCULATOR->getFunctionById(FUNCTION_ID_UNCERTAINTY)->referenceName());
 										str += ", ";
 										str += (char*) uncertainty;
 										str += ", 1)";
@@ -751,9 +786,9 @@ int DataSet::saveObjects(const char *file_name, bool save_global) {
 		filename = file_name;
 	}
 	const string *vstr;
-	xmlDocPtr doc = xmlNewDoc((xmlChar*) "1.0");	
-	xmlNodePtr cur, newnode, newnode2;	
-	doc->children = xmlNewDocNode(doc, NULL, (xmlChar*) "QALCULATE", NULL);	
+	xmlDocPtr doc = xmlNewDoc((xmlChar*) "1.0");
+	xmlNodePtr cur, newnode, newnode2;
+	doc->children = xmlNewDocNode(doc, NULL, (xmlChar*) "QALCULATE", NULL);
 	xmlNewProp(doc->children, (xmlChar*) "version", (xmlChar*) VERSION);
 	cur = doc->children;
 	DataObject *o;
@@ -785,7 +820,7 @@ int DataSet::saveObjects(const char *file_name, bool save_global) {
 				} else {
 					vstr = &o->getProperty(properties[i2], &approx);
 				}
-				if(save_global || approx >= 0 || !properties[i2]->isKey()) {	
+				if(save_global || approx >= 0 || !properties[i2]->isKey()) {
 					if(!vstr->empty()) {
 						if(properties[i2]->getReferenceName().find(' ') != string::npos) {
 							if(save_global && properties[i2]->propertyType() == PROPERTY_STRING) {
@@ -826,7 +861,7 @@ bool DataSet::objectsLoaded() const {
 void DataSet::setObjectsLoaded(bool objects_loaded) {
 	b_loaded = objects_loaded;
 }
-	
+
 void DataSet::addProperty(DataProperty *dp) {
 	properties.push_back(dp);
 	setChanged(true);
@@ -950,7 +985,7 @@ DataObject *DataSet::getNextObject(DataObjectIter *it) {
 	if(*it != objects.end()) return **it;
 	return NULL;
 }
-	
+
 const MathStructure *DataSet::getObjectProperyStruct(string property, string object) {
 	DataObject *o = getObject(object);
 	DataProperty *dp = getProperty(property);
@@ -983,7 +1018,7 @@ string DataSet::getObjectPropertyDisplayString(string property, string object) {
 	}
 	return empty_string;
 }
-	
+
 string DataSet::printProperties(string object) {
 	return printProperties(getObject(object));
 }
