@@ -1,5 +1,5 @@
 /*
-    Qalculate    
+    Qalculate
 
     Copyright (C) 2008  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
@@ -18,13 +18,14 @@
 #include "Variable.h"
 #include "Number.h"
 #include "Unit.h"
+#include "BuiltinFunctions.h"
 
 #include <limits.h>
 
 #if HAVE_UNORDERED_MAP
 #	include <unordered_map>
+	using std::unordered_map;
 #elif 	defined(__GNUC__)
-
 #	ifndef __has_include
 #	define __has_include(x) 0
 #	endif
@@ -52,6 +53,14 @@
 #	define unordered_map Sgi::hash_map
 #endif
 
+#include "MathStructure-support.h"
+
+using std::string;
+using std::vector;
+using std::iterator;
+using std::cout;
+using std::endl;
+
 class MathFunction_p {
 	public:
 		unordered_map<size_t, Argument*> argdefs;
@@ -59,6 +68,7 @@ class MathFunction_p {
 
 MathFunction::MathFunction(string name_, int argc_, int max_argc_, string cat_, string title_, string descr_, bool is_active) : ExpressionItem(cat_, name_, title_, descr_, false, true, is_active) {
 	priv = new MathFunction_p;
+	// arc = min number of arguments, max_argc = max arguments (if max_argc=-1, place additional arguments in vector)
 	argc = argc_;
 	if(max_argc_ < 0 || argc < 0) {
 		if(argc < 0) argc = 0;
@@ -67,6 +77,7 @@ MathFunction::MathFunction(string name_, int argc_, int max_argc_, string cat_,
 		max_argc = argc;
 	} else {
 		max_argc = max_argc_;
+		// arguments not required, must have a default value (by default 0)
 		for(int i = 0; i < max_argc - argc; i++) {
 			default_values.push_back("0");
 		}
@@ -114,11 +125,17 @@ int MathFunction::type() const {
 int MathFunction::subtype() const {
 	return SUBTYPE_FUNCTION;
 }
+int MathFunction::id() const {
+	return 0;
+}
 
 string MathFunction::example(bool raw_format, string name_string) const {
+	// example text
 	if(raw_format) return sexample;
 	string str = sexample;
+	// $name is replaced function name (or provided string)
 	gsub("$name", name_string.empty() ? name() : name_string, str);
+	// adjust decimal and group separators
 	return CALCULATOR->localizeExpression(str);
 }
 void MathFunction::setExample(string new_example) {
@@ -151,16 +168,19 @@ void MathFunction::setCondition(string expression) {
 	remove_blank_ends(scondition);
 }
 bool MathFunction::testCondition(const MathStructure &vargs) {
+	// test condition for arguments
 	if(scondition.empty()) {
 		return true;
 	}
+	// create a temporary function from the condition expression (for handling av arguments)
 	UserFunction test_function("", "CONDITION_TEST_FUNCTION", scondition, false, argc, "", "", max_argc);
 	MathStructure vargs2(vargs);
 	MathStructure mstruct(test_function.MathFunction::calculate(vargs2));
 	EvaluationOptions eo;
 	eo.approximation = APPROXIMATION_APPROXIMATE;
 	mstruct.eval(eo);
-	if(!mstruct.isNumber() || !mstruct.number().isPositive()) {
+	// check if result is true
+	if(!mstruct.isNumber() || !mstruct.number().getBoolean()) {
 		if(CALCULATOR->showArgumentErrors() && !CALCULATOR->aborted()) {
 			CALCULATOR->error(true, _("%s() requires that %s"), name().c_str(), printCondition().c_str(), NULL);
 		}
@@ -169,7 +189,8 @@ bool MathFunction::testCondition(const MathStructure &vargs) {
 	return true;
 }
 string MathFunction::printCondition() {
-	if(scondition.empty() || last_argdef_index == 0) return scondition;
+	if(scondition.empty()) return scondition;
+	// replace arguments (represented by \x, \y, \z, \a, etc.) with argument definition names
 	string str = scondition;
 	string svar, argstr;
 	Argument *arg;
@@ -181,7 +202,7 @@ string MathFunction::printCondition() {
 		svar = '\\';
 		if(maxargs() < 0 && i >= minargs()) {
 			svar += (char) ('v' + i - minargs());
-		} else { 
+		} else {
 			if('x' + i > 'z') {
 				svar += (char) ('a' + i - 3);
 			} else {
@@ -218,6 +239,7 @@ string MathFunction::printCondition() {
 	return str;
 }
 int MathFunction::args(const string &argstr, MathStructure &vargs, const ParseOptions &parseoptions) {
+	// read arguments from expression (e.g. "52, 2" from expression "sin(52, 2)"); used in Calculator::parse()
 	ParseOptions po = parseoptions;
 	MathStructure *unended_function = po.unended_function;
 	po.unended_function = NULL;
@@ -232,10 +254,12 @@ int MathFunction::args(const string &argstr, MathStructure &vargs, const ParseOp
 	bool last_is_vctr = false, vctr_started = false;
 	if(maxargs() > 0) {
 		arg = getArgumentDefinition(maxargs());
+		// if last argument is vector or maximum number of arguments is 1 and the only argument allows vector
 		last_is_vctr = arg && ((arg->type() == ARGUMENT_TYPE_VECTOR) || (maxargs() == 1 && arg->handlesVector()));
 	}
 	for(size_t str_index = 0; str_index < str.length(); str_index++) {
 		switch(str[str_index]) {
+			// argument does not end within parentheses, brackets or quotes
 			case LEFT_VECTOR_WRAP_CH: {}
 			case LEFT_PARENTHESIS_CH: {
 				if(!in_cit1 && !in_cit2) {
@@ -266,18 +290,24 @@ int MathFunction::args(const string &argstr, MathStructure &vargs, const ParseOp
 				}
 				break;
 			}
+			// argument separator
 			case COMMA_CH: {
 				if(pars == 0 && !in_cit1 && !in_cit2) {
 					itmp++;
+					// read one argument
 					if(itmp <= maxargs() || args() < 0) {
+						// index is <= max number of arguments
 						stmp = str.substr(start_pos, str_index - start_pos);
 						remove_blank_ends(stmp);
 						arg = getArgumentDefinition(itmp);
-						if(!arg && itmp > argc && args() < 0 && itmp > (int) last_argdef_index && last_argdef_index > 0) {
+						// index is greater than minimum number of arguments,
+						// and maximimum number of arguments is unlimited, use the last argument definition
+						if(!arg && itmp > argc && args() < 0 && itmp > (int) last_argdef_index && (int) last_argdef_index > argc) {
 							 arg = priv->argdefs[last_argdef_index];
 						}
 						if(stmp.empty()) {
 							if(arg) {
+								// if index has argument definition, use for parsing
 								MathStructure *mstruct = new MathStructure();
 								arg->parse(mstruct, getDefaultValue(itmp), po);
 								vargs.addChild_nocopy(mstruct);
@@ -298,6 +328,7 @@ int MathFunction::args(const string &argstr, MathStructure &vargs, const ParseOp
 							}
 						}
 					} else if(last_is_vctr) {
+						// if last argument is a vector, use additional arguments to fill the vector
 						if(!vctr_started) {
 							if(!vargs[vargs.size() - 1].isVector() || vargs[vargs.size() - 1].size() != 1) {
 								vargs[vargs.size() - 1].transform(STRUCT_VECTOR);
@@ -325,6 +356,9 @@ int MathFunction::args(const string &argstr, MathStructure &vargs, const ParseOp
 			}
 		}
 	}
+	// generate unended function information
+	// used from Calculator::parse() for display of continuous parsing info
+	// handling is incomplete if used separately
 	if(!str.empty()) {
 		itmp++;
 		po.unended_function = unended_function;
@@ -332,7 +366,7 @@ int MathFunction::args(const string &argstr, MathStructure &vargs, const ParseOp
 			stmp = str.substr(start_pos, str.length() - start_pos);
 			remove_blank_ends(stmp);
 			arg = getArgumentDefinition(itmp);
-			if(!arg && itmp > argc && args() < 0 && itmp > (int) last_argdef_index && last_argdef_index > 0) {
+			if(!arg && itmp > argc && args() < 0 && itmp > (int) last_argdef_index && (int) last_argdef_index > argc) {
 				 arg = priv->argdefs[last_argdef_index];
 			}
 			if(stmp.empty()) {
@@ -345,7 +379,7 @@ int MathFunction::args(const string &argstr, MathStructure &vargs, const ParseOp
 					CALCULATOR->parse(mstruct, getDefaultValue(itmp));
 					vargs.addChild_nocopy(mstruct);
 				}
-			} else {				
+			} else {
 				if(arg) {
 					MathStructure *mstruct = new MathStructure();
 					arg->parse(mstruct, stmp, po);
@@ -387,6 +421,7 @@ int MathFunction::args(const string &argstr, MathStructure &vargs, const ParseOp
 			unended_function->addChild(m_undefined);
 		}
 	}
+	// append default values
 	if(itmp < maxargs() && itmp >= minargs()) {
 		int itmp2 = itmp;
 		while(itmp2 < maxargs()) {
@@ -471,7 +506,7 @@ MathStructure MathFunction::createFunctionMathStructureFromVArgs(const MathStruc
 	return mstruct;
 }
 MathStructure MathFunction::createFunctionMathStructureFromSVArgs(vector<string> &svargs) {
-	MathStructure mstruct(this, NULL); 
+	MathStructure mstruct(this, NULL);
 	for(size_t i = 0; i < svargs.size(); i++) {
 		mstruct.addChild(svargs[i]);
 	}
@@ -487,7 +522,7 @@ MathStructure MathFunction::calculate(const string &argv, const EvaluationOption
 }
 MathStructure MathFunction::parse(const string &argv, const ParseOptions &po) {
 	MathStructure vargs;
-	args(argv, vargs, po);	
+	args(argv, vargs, po);
 	vargs.setType(STRUCT_FUNCTION);
 	vargs.setFunction(this);
 	return vargs;
@@ -505,8 +540,24 @@ bool MathFunction::testArguments(MathStructure &vargs) {
 		if(it->first > last) {
 			last = it->first;
 		}
-		if(it->second && it->first > 0 && it->first <= vargs.size() && !it->second->test(vargs[it->first - 1], it->first, this)) {
-			return false;
+		if(it->second && it->first > 0 && it->first <= vargs.size()) {
+			// for symbols arguments with zero or undefined value, search the first argument for a symbol
+			if(it->second->type() == ARGUMENT_TYPE_SYMBOLIC && (vargs[it->first - 1].isZero() || vargs[it->first - 1].isUndefined())) {
+				vargs[it->first - 1] = vargs[0].find_x_var();
+				if(vargs[it->first - 1].isUndefined() && vargs[0].isVariable() && vargs[0].variable()->isKnown()) vargs[it->first - 1] = ((KnownVariable*) vargs[0].variable())->get().find_x_var();
+				if(vargs[it->first - 1].isUndefined()) {
+					CALCULATOR->beginTemporaryStopMessages();
+					MathStructure mtest(vargs[0]);
+					mtest.eval();
+					vargs[it->first - 1] = mtest.find_x_var();
+					CALCULATOR->endTemporaryStopMessages();
+				}
+				if(vargs[it->first - 1].isUndefined()) {
+					vargs[it->first - 1].set(CALCULATOR->getVariableById(VARIABLE_ID_X), true);
+					CALCULATOR->error(false, _("No unknown variable/symbol was found."), NULL);
+				}
+			}
+			if(!it->second->test(vargs[it->first - 1], it->first, this)) return false;
 		}
 	}
 	if(max_argc < 0 && (int) last > argc && priv->argdefs.find(last) != priv->argdefs.end()) {
@@ -619,8 +670,8 @@ int MathFunction::stringArgs(const string &argstr, vector<string> &svargs) {
 					itmp++;
 					if(itmp <= maxargs() || args() < 0) {
 						stmp = str.substr(start_pos, str_index - start_pos);
-						remove_blank_ends(stmp);																				
-						remove_parenthesis(stmp);						
+						remove_blank_ends(stmp);
+						remove_parenthesis(stmp);
 						remove_blank_ends(stmp);
 						if(stmp.empty()) {
 							stmp = getDefaultValue(itmp);
@@ -637,26 +688,26 @@ int MathFunction::stringArgs(const string &argstr, vector<string> &svargs) {
 		itmp++;
 		if(itmp <= maxargs() || args() < 0) {
 			stmp = str.substr(start_pos, str.length() - start_pos);
-			remove_blank_ends(stmp);																				
-			remove_parenthesis(stmp);						
+			remove_blank_ends(stmp);
+			remove_parenthesis(stmp);
 			remove_blank_ends(stmp);
 			if(stmp.empty()) {
 				stmp = getDefaultValue(itmp);
 			}
 			svargs.push_back(stmp);
-		}	
+		}
 	}
 	if(itmp < maxargs() && itmp >= minargs()) {
 		int itmp2 = itmp;
 		while(itmp2 < maxargs()) {
-			svargs.push_back(default_values[itmp2 - minargs()]);	
+			svargs.push_back(default_values[itmp2 - minargs()]);
 			itmp2++;
 		}
 	}
 	return itmp;
 }
 
-MathStructure MathFunction::produceVector(const MathStructure &vargs, int begin, int end) {	
+MathStructure MathFunction::produceVector(const MathStructure &vargs, int begin, int end) {
 	if(begin < 1) {
 		begin = minargs() + 1;
 		if(begin < 1) begin = 1;
@@ -676,7 +727,7 @@ MathStructure MathFunction::produceVector(const MathStructure &vargs, int begin,
 	MathStructure mstruct2;
 	return mstruct.flattenVector(mstruct2);
 }
-MathStructure MathFunction::produceArgumentsVector(const MathStructure &vargs, int begin, int end) {	
+MathStructure MathFunction::produceArgumentsVector(const MathStructure &vargs, int begin, int end) {
 	if(begin < 1) {
 		begin = minargs() + 1;
 		if(begin < 1) begin = 1;
@@ -748,9 +799,10 @@ int UserFunction::subtype() const {
 }
 extern string format_and_print(const MathStructure &mstruct);
 bool replace_intervals_f(MathStructure &mstruct) {
+	// replace intervals with temporary variables
 	if(mstruct.isNumber() && (mstruct.number().isInterval(false) || (CALCULATOR->usesIntervalArithmetic() && mstruct.number().precision() >= 0))) {
 		Variable *v = new KnownVariable("", format_and_print(mstruct), mstruct);
-		v->ref();
+		v->setTitle("\b");
 		mstruct.set(v, true);
 		v->destroy();
 		return true;
@@ -764,9 +816,9 @@ bool replace_intervals_f(MathStructure &mstruct) {
 	}
 	return b;
 }
-extern bool create_interval(MathStructure &mstruct, const MathStructure &m1, const MathStructure &m2);
 bool replace_f_interval(MathStructure &mstruct, const EvaluationOptions &eo) {
-	if(mstruct.isFunction() && mstruct.function() == CALCULATOR->f_interval && mstruct.size() == 2) {
+	// replace interval() and uncertainty() with numbers with intervals, if possible
+	if(mstruct.isFunction() && mstruct.function()->id() == FUNCTION_ID_INTERVAL && mstruct.size() == 2) {
 		if(mstruct[0].isNumber() && mstruct[1].isNumber()) {
 			Number nr;
 			if(nr.setInterval(mstruct[0].number(), mstruct[1].number())) {
@@ -782,6 +834,56 @@ bool replace_f_interval(MathStructure &mstruct, const EvaluationOptions &eo) {
 			if(create_interval(mstruct, m1, m2)) return true;
 		}
 		return false;
+	} else if(eo.interval_calculation != INTERVAL_CALCULATION_NONE && mstruct.isFunction() && mstruct.function()->id() == FUNCTION_ID_UNCERTAINTY && mstruct.size() == 3) {
+		if(mstruct[0].isNumber() && mstruct[1].isNumber()) {
+			Number nr(mstruct[0].number());
+			if(mstruct[2].number().getBoolean()) {
+				nr.setRelativeUncertainty(mstruct[1].number(), false);
+			} else {
+				nr.setUncertainty(mstruct[1].number(), false);
+			}
+			mstruct.set(nr, true);
+			return true;
+		} else {
+			MathStructure m1(mstruct[0]);
+			MathStructure m2(mstruct[1]);
+			if(mstruct[2].number().getBoolean()) {
+				m1.eval(eo);
+				m2.eval(eo);
+				if(m1.isNumber() && m2.isNumber()) {
+					Number nr(m1.number());
+					nr.setRelativeUncertainty(m2.number(), false);
+					mstruct.set(nr, true);
+					return true;
+				}
+				m1 = mstruct[0];
+				m2 = mstruct[1];
+				mstruct.setToChild(1, true);
+				mstruct *= m_one;
+				mstruct.last() -= m2;
+				mstruct.transformById(FUNCTION_ID_INTERVAL);
+				m1 *= m_one;
+				m1.last() += m2;
+				mstruct.addChild(m1);
+				return 1;
+			} else {
+				if(set_uncertainty(m1, m2, eo, false)) return true;
+				m1.eval(eo);
+				m2.eval(eo);
+				if(set_uncertainty(m1, m2, eo, true)) return true;
+				m1 = mstruct[0];
+				m2 = mstruct[1];
+				mstruct.setToChild(1);
+				mstruct -= m2;
+				mstruct.transformById(FUNCTION_ID_INTERVAL);
+				m1 += m2;
+				mstruct.addChild(m1);
+			}
+			replace_f_interval(mstruct, eo);
+			return true;
+
+		}
+		return false;
 	}
 	bool b = false;
 	for(size_t i = 0; i < mstruct.size(); i++) {
@@ -806,16 +908,52 @@ int UserFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 		if(i_args < 0) {
 			i_args = minargs();
 		}
-		
+
 		for(int i = 0; i < i_args; i++) {
-			if(vargs[i].containsInterval(true) || vargs[i].containsFunction(CALCULATOR->f_interval, true)) {
-				MathStructure *mv = new MathStructure(vargs[i]);
-				replace_f_interval(*mv, eo);
-				replace_intervals_f(*mv);
-				v_id.push_back(CALCULATOR->addId(mv, true));
-			} else {
-				v_id.push_back(CALCULATOR->addId(new MathStructure(vargs[i]), true));
+			MathStructure *mv = new MathStructure(vargs[i]);
+			Argument *arg = getArgumentDefinition(i + 1);
+			if((!arg || !arg->tests() || arg->type() == ARGUMENT_TYPE_FREE) && mv->containsInterval(true, false, false, 0, true)) {
+				size_t count = 0;
+				for(size_t i3 = 0; i3 < 1 || (maxargs() < 0 && i3 < 3); i3++) {
+					svar = '\\';
+					if(i3 == 1) {
+						svar += 'v';
+					} else if(i3 == 2) {
+						svar += 'w';
+					} else if('x' + i > 'z') {
+						svar += (char) ('a' + i - 3);
+					} else {
+						svar += 'x' + i;
+					}
+					size_t pos = 0;
+					while((pos = sformula_calc.find(svar, pos)) != string::npos) {
+						pos += svar.length();
+						count++;
+					}
+					for(size_t i2 = 0; i2 < v_subs.size(); i2++) {
+						pos = 0;
+						size_t c2 = 0;
+						while((pos = v_subs[i2].find(svar, pos)) != string::npos) {
+							pos += svar.length();
+							c2++;
+						}
+						string svar2 = "\\";
+						svar2 += i2s(i2 + 1);
+						pos = 0;
+						size_t c_sub = 0;
+						while((pos = sformula_calc.find(svar2, pos)) != string::npos) {
+							pos += svar2.length();
+							c_sub++;
+						}
+						count += c2 * c_sub;
+					}
+				}
+				if(count > 1) {
+					replace_f_interval(*mv, eo);
+					replace_intervals_f(*mv);
+				}
 			}
+			v_id.push_back(CALCULATOR->addId(mv, true));
 			v_strs.push_back(LEFT_PARENTHESIS ID_WRAP_LEFT);
 			v_strs[i] += i2s(v_id[i]);
 			v_strs[i] += ID_WRAP_RIGHT RIGHT_PARENTHESIS;
@@ -845,7 +983,7 @@ int UserFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 					} else {
 						svar += 'x' + i3;
 					}
-					i2 = 0;	
+					i2 = 0;
 					while(true) {
 						if((i2 = str.find(svar, i2)) != string::npos) {
 							if(i2 != 0 && str[i2 - 1] == '\\') {
@@ -861,7 +999,7 @@ int UserFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 				if(maxargs() < 0) {
 					i2 = 0;
 					while(true) {
-						if((i2 = str.find("\\v")) != string::npos) {					
+						if((i2 = str.find("\\v")) != string::npos) {
 							if(i2 != 0 && str[i2 - 1] == '\\') {
 								i2 += 2;
 							} else {
@@ -873,7 +1011,7 @@ int UserFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 					}
 					i2 = 0;
 					while(true) {
-						if((i2 = str.find("\\w")) != string::npos) {					
+						if((i2 = str.find("\\w")) != string::npos) {
 							if(i2 != 0 && str[i2 - 1] == '\\') {
 								i2 += 2;
 							} else {
@@ -882,7 +1020,7 @@ int UserFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 						} else {
 							break;
 						}
-					}			
+					}
 				}
 				MathStructure *v_mstruct = new MathStructure();
 				CALCULATOR->parse(v_mstruct, str, po);
@@ -914,7 +1052,7 @@ int UserFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 						if(i2 != 0 && stmp[i2 - 1] == '\\') {
 							i2 += svar.size();
 						} else {
-							stmp.replace(i2, svar.size(), v_subs[i]);
+							stmp.replace(i2, svar.size(), string("(") + v_subs[i] + ")");
 						}
 					} else {
 						break;
@@ -929,7 +1067,7 @@ int UserFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 			} else {
 				svar += 'x' + i;
 			}
-			i2 = 0;	
+			i2 = 0;
 			while(true) {
 				if((i2 = stmp.find(svar, i2)) != string::npos) {
 					if(i2 != 0 && stmp[i2 - 1] == '\\') {
@@ -945,7 +1083,7 @@ int UserFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 		if(maxargs() < 0) {
 			i2 = 0;
 			while(true) {
-				if((i2 = stmp.find("\\v")) != string::npos) {					
+				if((i2 = stmp.find("\\v")) != string::npos) {
 					if(i2 != 0 && stmp[i2 - 1] == '\\') {
 						i2 += 2;
 					} else {
@@ -957,7 +1095,7 @@ int UserFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 			}
 			i2 = 0;
 			while(true) {
-				if((i2 = stmp.find("\\w")) != string::npos) {					
+				if((i2 = stmp.find("\\w")) != string::npos) {
 					if(i2 != 0 && stmp[i2 - 1] == '\\') {
 						i2 += 2;
 					} else {
@@ -966,7 +1104,7 @@ int UserFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 				} else {
 					break;
 				}
-			}			
+			}
 		}
 		while(true) {
 			if((i2 = stmp.find("\\\\")) != string::npos) {
@@ -1017,13 +1155,13 @@ void UserFunction::setFormula(string new_formula, int argc_, int max_argc_) {
 				svar_o += (char) ('A' + i - 3);
 			else
 				svar_o += 'X' + i;
-				
+
 			before_find_in_set_formula:
 			if(i < 24 && (i2 = new_formula.find(svar_o, i4)) != string::npos) {
 				if(i2 > 0 && new_formula[i2 - 1] == '\\') {
 					i4 = i2 + 2;
 					goto before_find_in_set_formula;
-				}				
+				}
 				i3 = 0;
 				if(new_formula.length() > i2 + 2 && new_formula[i2 + 2] == ID_WRAP_LEFT_CH) {
 					if((i3 = new_formula.find(ID_WRAP_RIGHT_CH, i2 + 2)) != string::npos) {
@@ -1068,11 +1206,11 @@ void UserFunction::setFormula(string new_formula, int argc_, int max_argc_) {
 						if(i2 > 0 && v_subs[sub_i][i2 - 1] == '\\') {
 							i4 = i2 + 2;
 							goto before_find_in_vsubs_set_formula;
-						}				
+						}
 						i3 = 0;
 						if(v_subs[sub_i].length() > i2 + 2 && v_subs[sub_i][i2 + 2] == ID_WRAP_LEFT_CH) {
 							if((i3 = v_subs[sub_i].find(ID_WRAP_RIGHT_CH, i2 + 2)) != string::npos) {
-								svar_v = v_subs[sub_i].substr(i2 + 3, i3 - (i2 + 3));	
+								svar_v = v_subs[sub_i].substr(i2 + 3, i3 - (i2 + 3));
 								i3 -= i2 + 1;
 							} else i3 = 0;
 						}
@@ -1115,7 +1253,7 @@ void UserFunction::setFormula(string new_formula, int argc_, int max_argc_) {
 				} else {
 					max_argc_++;
 					argc_++;
-				}			
+				}
 			}
 		}
 	}
@@ -1129,16 +1267,16 @@ void UserFunction::setFormula(string new_formula, int argc_, int max_argc_) {
 		max_argc_ = -1;
 		if(argc_ < 0) argc_ = 0;
 	} else if(max_argc_ < argc_) {
-		max_argc_ = argc_;	
+		max_argc_ = argc_;
 	}
-	
+
 	while((int) default_values.size() < max_argc_ - argc_) {
 		default_values.push_back("0");
 	}
 	if(max_argc_ > 0) default_values.resize(max_argc_ - argc_);
 	sformula_calc = new_formula;
 	argc = argc_;
-	max_argc = max_argc_;	
+	max_argc = max_argc_;
 }
 void UserFunction::addSubfunction(string subfunction, bool precalculate) {
 	setChanged(true);
@@ -1224,7 +1362,7 @@ string Argument::printlong() const {
 		}
 		str += " ";
 		str += _("that is rational (polynomial)");
-	} 
+	}
 	if(!scondition.empty()) {
 		if(!b_zero || b_rational) {
 			str += " ";
@@ -1234,7 +1372,7 @@ string Argument::printlong() const {
 		str += _("that fulfills the condition:");
 		str += " \"";
 		string str2 = scondition;
-		if(name().empty()) gsub("\\x", _("Argument"), str2);
+		if(name().empty()) gsub("\\x", _("argument"), str2);
 		else gsub("\\x", name(), str2);
 		str += str2;
 		str += "\"";
@@ -1259,14 +1397,14 @@ bool Argument::test(MathStructure &value, int index, MathFunction *f, const Eval
 	bool b = subtest(value, eo);
 	if(b && !b_zero) {
 		if(!value.isNumber() && !value.representsNonZero()) {
-			value.eval(eo);	
+			value.eval(eo);
 			evaled = true;
 		}
 		b = value.representsNonZero();
 	}
 	if(b && b_rational) {
 		if(!evaled) {
-			value.eval(eo);	
+			value.eval(eo);
 			evaled = true;
 		}
 		b = value.isRationalPolynomial();
@@ -1296,7 +1434,7 @@ bool Argument::test(MathStructure &value, int index, MathFunction *f, const Eval
 		if(value.isVector()) return false;
 	}
 	if(!b) {
-		if(b_error) {
+		if(b_error && (type() != ARGUMENT_TYPE_SYMBOLIC || !value.isUndefined())) {
 			if(sname.empty()) {
 				CALCULATOR->error(true, _("Argument %s in %s() must be %s."), i2s(index).c_str(), f->name().c_str(), printlong().c_str(), NULL);
 			} else {
@@ -1350,7 +1488,7 @@ void Argument::parse(MathStructure *mstruct, const string &str, const ParseOptio
 				CALCULATOR->parse(mstruct, str.substr(1 + pars, str.length() - 2 - pars * 2), po);
 				if(mstruct->isDateTime() && !mstruct->datetime()->parsed_string.empty()) mstruct->set(mstruct->datetime()->parsed_string, false, true);
 				return;
-			}	
+			}
 			if((str[pars] == '\"' && str[str.length() - 1 - pars] == '\"') || (str[pars] == '\'' && str[str.length() - 1 - pars] == '\'')) {
 				size_t i = pars + 1, cits = 0;
 				while(i < str.length() - 1 - pars) {
@@ -1379,7 +1517,7 @@ void Argument::parse(MathStructure *mstruct, const string &str, const ParseOptio
 						str3 = "(";
 						if(!m_temp) {
 							CALCULATOR->error(true, _("Internal id %s does not exist."), i2s(id).c_str(), NULL);
-							str3 += CALCULATOR->v_undef->preferredInputName(true, false, false, true).name;
+							str3 += CALCULATOR->getVariableById(VARIABLE_ID_UNDEFINED)->preferredInputName(true, false, false, true).name;
 						} else {
 							str3 += m_temp->print(CALCULATOR->save_printoptions).c_str();
 							m_temp->unref();
@@ -1410,7 +1548,7 @@ void Argument::parse(MathStructure *mstruct, const string &str, const ParseOptio
 			str3 = "(";
 			if(!m_temp) {
 				CALCULATOR->error(true, _("Internal id %s does not exist."), i2s(id).c_str(), NULL);
-				str3 += CALCULATOR->v_undef->preferredInputName(true, false, false, true).name;
+				str3 += CALCULATOR->getVariableById(VARIABLE_ID_UNDEFINED)->preferredInputName(true, false, false, true).name;
 			} else {
 				str3 += m_temp->print(CALCULATOR->save_printoptions).c_str();
 				m_temp->unref();
@@ -1520,7 +1658,7 @@ NumberArgument::~NumberArgument() {
 		delete fmax;
 	}
 }
-	
+
 void NumberArgument::setMin(const Number *nmin) {
 	if(!nmin) {
 		if(fmin) {
@@ -1581,6 +1719,10 @@ bool NumberArgument::subtest(MathStructure &value, const EvaluationOptions &eo)
 	if(!value.isNumber()) {
 		value.eval(eo);
 	}
+	/*if(value.isMultiplication() && value.size() == 2 && value[0].isNumber() && value[1].isUnit() && value[1].unit()->baseUnit() == CALCULATOR->getRadUnit()->baseUnit()) {
+		value /= CALCULATOR->getRadUnit();
+		value.eval(eo);
+	}*/
 	if(!value.isNumber() || (b_rational_number && !value.number().isRational())) {
 		return false;
 	}
@@ -1628,7 +1770,7 @@ void NumberArgument::set(const Argument *arg) {
 		}
 		if(farg->max()) {
 			fmax = new Number(*farg->max());
-		}		
+		}
 	}
 	Argument::set(arg);
 }
@@ -1691,8 +1833,8 @@ IntegerArgument::IntegerArgument(string name_, ArgumentMinMaxPreDefinition minma
 		case ARGUMENT_MIN_MAX_NONZERO: {
 			setZeroForbidden(true);
 			break;
-		}	
-		default: {}	
+		}
+		default: {}
 	}
 	b_handle_vector = does_test;
 }
@@ -1763,7 +1905,7 @@ bool IntegerArgument::subtest(MathStructure &value, const EvaluationOptions &eo)
 		if(!(COMPARISON_IS_EQUAL_OR_LESS(cmpr))) {
 			return false;
 		}
-	}	
+	}
 	return true;
 }
 int IntegerArgument::type() const {
@@ -1833,9 +1975,9 @@ string IntegerArgument::subprintlong() const {
 		str += " ";
 		switch(i_inttype) {
 			case INTEGER_TYPE_SIZE: {}
-			case INTEGER_TYPE_UINT: {str += i2s(UINT_MAX); break;}
+			case INTEGER_TYPE_UINT: {str += u2s(UINT_MAX); break;}
 			case INTEGER_TYPE_SINT: {str += i2s(INT_MAX); break;}
-			case INTEGER_TYPE_ULONG: {str += i2s(ULONG_MAX); break;}
+			case INTEGER_TYPE_ULONG: {str += u2s(ULONG_MAX); break;}
 			case INTEGER_TYPE_SLONG: {str += i2s(LONG_MAX); break;}
 			default: {}
 		}
@@ -1901,12 +2043,12 @@ VectorArgument::VectorArgument(string name_, bool does_test, bool allow_matrix,
 VectorArgument::VectorArgument(const VectorArgument *arg) {
 	set(arg);
 	b_argloop = arg->reoccuringArguments();
-	size_t i = 1; 
+	size_t i = 1;
 	while(true) {
 		if(!arg->getArgument(i)) break;
 		subargs.push_back(arg->getArgument(i)->copy());
 		i++;
-	}	
+	}
 }
 VectorArgument::~VectorArgument() {
 	for(size_t i = 0; i < subargs.size(); i++) {
@@ -2108,6 +2250,7 @@ void AngleArgument::parse(MathStructure *mstruct, const string &str, const Parse
 		if(mstruct->contains(CALCULATOR->getRadUnit(), false, true, true) > 0) return;
 		if(mstruct->contains(CALCULATOR->getDegUnit(), false, true, true) > 0) return;
 		if(mstruct->contains(CALCULATOR->getGraUnit(), false, true, true) > 0) return;
+		if(contains_angle_unit(*mstruct, po)) return;
 	}
 	switch(po.angle_unit) {
 		case ANGLE_UNIT_DEGREES: {
@@ -2129,7 +2272,7 @@ void AngleArgument::parse(MathStructure *mstruct, const string &str, const Parse
 ArgumentSet::ArgumentSet(string name_, bool does_test, bool does_error) : Argument(name_, does_test, does_error) {
 }
 ArgumentSet::ArgumentSet(const ArgumentSet *arg) {
-	set(arg); 
+	set(arg);
 	size_t i = 1;
 	while(true) {
 		if(!arg->getArgument(i)) break;
