@@ -16,6 +16,10 @@
 #include "Calculator.h"
 #include "MathStructure.h"
 #include "Prefix.h"
+#include "BuiltinFunctions.h"
+
+using std::string;
+using std::vector;
 
 Unit::Unit(string cat_, string name_, string plural_, string singular_, string title_, bool is_local, bool is_builtin, bool is_active) : ExpressionItem(cat_, "", title_, "", is_local, is_builtin, is_active) {
 	remove_blank_ends(plural_);
@@ -113,7 +117,7 @@ void Unit::setUseWithPrefixesByDefault(bool use_with_prefixes) {
 	b_use_with_prefixes = use_with_prefixes;
 }
 bool Unit::isCurrency() const {
-	return baseUnit() == CALCULATOR->u_euro;
+	return baseUnit() == CALCULATOR->getUnitById(UNIT_ID_EURO);
 }
 const string &Unit::countries() const {return scountries;}
 void Unit::setCountries(string country_names) {
@@ -249,16 +253,19 @@ bool Unit::convert(Unit *u, MathStructure &mvalue, MathStructure &mexp) const {
 	} else if(u->baseUnit() == baseUnit()) {
 		u->convertToBaseUnit(mvalue, mexp);
 		convertFromBaseUnit(mvalue, mexp);
-		if(isCurrency() && u->isCurrency() && ((isBuiltin() && this != CALCULATOR->u_euro) || (u->isBuiltin() && u != CALCULATOR->u_euro))) {
+		if(isCurrency() && u->isCurrency() && ((isBuiltin() && this != CALCULATOR->getUnitById(UNIT_ID_EURO)) || (u->isBuiltin() && u != CALCULATOR->getUnitById(UNIT_ID_EURO)))) {
 			int i = 1;
-			if(u == CALCULATOR->u_btc || this == CALCULATOR->u_btc) i = 2;
-			if(u->subtype() == SUBTYPE_ALIAS_UNIT) {
-				if(i < 2 && ((AliasUnit*) u)->firstBaseUnit() == CALCULATOR->u_btc) i = 2;
-				else if(((AliasUnit*) u)->firstBaseUnit() != CALCULATOR->u_euro) i = 3;
+			if(i < 4 && (u == CALCULATOR->getUnitById(UNIT_ID_BYN) || this == CALCULATOR->getUnitById(UNIT_ID_BYN))) i = (i == 3 ? 5 : 4);
+			if(i < 2 && (u == CALCULATOR->getUnitById(UNIT_ID_BTC) || this == CALCULATOR->getUnitById(UNIT_ID_BTC))) i = 2;
+			if(i < 5 && u->subtype() == SUBTYPE_ALIAS_UNIT) {
+				if(i < 2 && ((AliasUnit*) u)->firstBaseUnit() == CALCULATOR->getUnitById(UNIT_ID_BTC)) i = 2;
+				else if(i < 5 && ((AliasUnit*) u)->firstBaseUnit() != CALCULATOR->getUnitById(UNIT_ID_EURO)) i = (i == 4 ? 5 : 3);
+				else if(i < 4 && ((AliasUnit*) u)->firstBaseUnit() == CALCULATOR->getUnitById(UNIT_ID_BYN)) i = (i == 3 ? 5 : 4);
 			}
-			if(i < 3 && subtype() == SUBTYPE_ALIAS_UNIT) {
-				if(i < 2 && ((AliasUnit*) this)->firstBaseUnit() == CALCULATOR->u_btc) i = 2;
-				else if(((AliasUnit*) this)->firstBaseUnit() != CALCULATOR->u_euro) i = 3;
+			if(i < 5 && subtype() == SUBTYPE_ALIAS_UNIT) {
+				if(i < 2 && ((AliasUnit*) this)->firstBaseUnit() == CALCULATOR->getUnitById(UNIT_ID_BTC)) i = 2;
+				else if(i < 5 && ((AliasUnit*) this)->firstBaseUnit() != CALCULATOR->getUnitById(UNIT_ID_EURO)) i = (i == 4 ? 5 : 3);
+				else if(i < 4 && ((AliasUnit*) this)->firstBaseUnit() == CALCULATOR->getUnitById(UNIT_ID_BYN)) i = (i == 3 ? 5 : 4);
 			}
 			CALCULATOR->setExchangeRatesUsed(i);
 		}
@@ -373,7 +380,7 @@ MathStructure &AliasUnit::convertFromBaseUnit(MathStructure &mvalue, MathStructu
 		u = u2;
 		u2->convertFromFirstBaseUnit(mvalue, mexp);
 		if(u == this) break;
-	}	
+	}
 	return mvalue;
 }
 MathStructure &AliasUnit::convertToBaseUnit(MathStructure &mvalue) const {
@@ -454,7 +461,7 @@ MathStructure &AliasUnit::convertFromFirstBaseUnit(MathStructure &mvalue, MathSt
 			} else {
 				CALCULATOR->parse(mstruct, svalue, po);
 				if(mstruct->containsType(STRUCT_UNIT, false, true, true)) {
-					mstruct->transform(CALCULATOR->f_stripunits);
+					mstruct->transformById(FUNCTION_ID_STRIP_UNITS);
 				}
 			}
 			if(!suncertainty.empty()) {
@@ -539,7 +546,7 @@ MathStructure &AliasUnit::convertFromFirstBaseUnit(MathStructure &mvalue, MathSt
 			if(!mexp.isOne()) mstruct->raise(mexp);
 			mvalue.multiply_nocopy(mstruct, true);
 		}
-	}	
+	}
 	return mvalue;
 }
 MathStructure &AliasUnit::convertToFirstBaseUnit(MathStructure &mvalue, MathStructure &mexp) const {
@@ -594,7 +601,7 @@ MathStructure &AliasUnit::convertToFirstBaseUnit(MathStructure &mvalue, MathStru
 		} else {
 			CALCULATOR->parse(mstruct, svalue, po);
 			if(mstruct->containsType(STRUCT_UNIT, false, true, true) > 0) {
-				mstruct->transform(CALCULATOR->f_stripunits);
+				mstruct->transformById(FUNCTION_ID_STRIP_UNITS);
 			}
 		}
 		if(!suncertainty.empty()) {
@@ -626,7 +633,7 @@ MathStructure &AliasUnit::convertToFirstBaseUnit(MathStructure &mvalue, MathStru
 			}
 		} else if(isApproximate() && !mstruct->isApproximate()) {
 			mstruct->setApproximate(true, true);
-		}		
+		}
 		if(!mexp.isOne()) mstruct->raise(mexp);
 		mvalue.multiply_nocopy(mstruct, true);
 	}
@@ -696,7 +703,7 @@ bool AliasUnit::hasNonlinearRelationTo(Unit *u) const {
 			if(fbu->subtype() != SUBTYPE_ALIAS_UNIT) return false;
 			if(((AliasUnit*) fbu)->hasNonlinearExpression()) return true;
 			fbu = (Unit*) ((AliasUnit*) fbu)->firstBaseUnit();
-		}	
+		}
 	} else if(isChildOf(u)) {
 		Unit *fbu = (Unit*) this;
 		while(true) {
@@ -704,7 +711,7 @@ bool AliasUnit::hasNonlinearRelationTo(Unit *u) const {
 			if(fbu->subtype() != SUBTYPE_ALIAS_UNIT) return false;
 			if(((AliasUnit*) fbu)->hasNonlinearExpression()) return true;
 			fbu = (Unit*) ((AliasUnit*) fbu)->firstBaseUnit();
-		}			
+		}
 	} else {
 		return hasNonlinearRelationTo(baseUnit()) || u->hasNonlinearRelationTo(u->baseUnit());
 	}
@@ -931,7 +938,7 @@ string CompositeUnit::print(bool plural_, bool short_, bool use_unicode, bool (*
 				if(i < units.size() - 1) {
 					b2 = true;
 					str += "(";
-				}				
+				}
 			} else {
 //				if(i > 0) str += "*";
 				if(i > 0) str += " ";
@@ -976,7 +983,7 @@ bool CompositeUnit::containsRelativeTo(Unit *u) const {
 			if(containsRelativeTo(cu->get(i)->baseUnit())) return true;
 		}
 		return false;
-	}	
+	}
 	return false;
 }
 bool CompositeUnit::hasNonlinearRelationToBase() const {
@@ -1007,8 +1014,8 @@ MathStructure CompositeUnit::generateMathStructure(bool make_division, bool set_
 		MathStructure mstruct2;
 		if(!has_p || units[i]->prefix()) {
 			mstruct2.set(units[i]->firstBaseUnit(), units[i]->prefix());
-		} else {				
-			mstruct2.set(units[i]->firstBaseUnit(), CALCULATOR->decimal_null_prefix);
+		} else {
+			mstruct2.set(units[i]->firstBaseUnit(), CALCULATOR->getDecimalNullPrefix());
 		}
 		if(make_division && units[i]->firstBaseExponent() < 0) {
 			if(units[i]->firstBaseExponent() != -1) {
@@ -1028,10 +1035,10 @@ MathStructure CompositeUnit::generateMathStructure(bool make_division, bool set_
 			if(mden.isZero()) {
 				mden = mstruct2;
 			} else {
-				mden *= mstruct2;
+				mden.multiply(mstruct2, true);
 			}
 		} else {
-			mstruct *= mstruct2;
+			mstruct.multiply(mstruct2, true);
 		}
 	}
 	if(make_division && !mden.isZero()) {
